/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SMatrix } from "./SMatrix";

/**
 * 鼠标事件
 *
 * @author 郝洁 <haojie@persagy.com>
 */
export class SMouseEvent {
    /** 事件类型 */
    type: string;
    /** x 坐标 */
    x: number;
    /** y 坐标 */
    y: number;
    /** 屏幕 x 坐标 */
    screenX: number;
    /** 屏幕 y 坐标 */
    screenY: number;
    /** 客户端 x 坐标 */
    clientX: number;
    /** 客户端 Y 坐标 */
    clientY: number;
    /** X 轴偏移量 */
    offsetX: number;
    /** Y 轴偏移量 */
    offsetY: number;
    /** 是否按下 alt 键 */
    altKey: boolean;
    /** 是否按下 ctrl 键 */
    ctrlKey: boolean;
    /** 是否按下 shiftKey 键 */
    shiftKey: boolean;
    /** 鼠标按键 */
    buttons: number;
    /** 变换矩阵 */
    matrix = new SMatrix();

    /**
     * 构造函数
     *
     * @param event     系统鼠标事件
     */
    constructor(event: MouseEvent | SMouseEvent) {
        // let bbox = (this.type = event.type); //event.srcElement.getBoundingClientRect()
        this.type = event.type;
        this.screenX = event.screenX;
        this.screenY = event.screenY;
        this.clientX = event.clientX;
        this.clientY = event.clientY;
        this.offsetX = event.offsetX;
        this.offsetY = event.offsetY;
        this.altKey = event.altKey;
        this.ctrlKey = event.ctrlKey;
        this.shiftKey = event.shiftKey;
        this.buttons = event.buttons;
        if (event instanceof SMouseEvent) {
            this.x = event.x;
            this.y = event.y;
            this.matrix.m11 = event.matrix.m11;
            this.matrix.m12 = event.matrix.m12;
            this.matrix.m13 = event.matrix.m13;
            this.matrix.m14 = event.matrix.m14;
            this.matrix.m21 = event.matrix.m21;
            this.matrix.m22 = event.matrix.m22;
            this.matrix.m23 = event.matrix.m23;
            this.matrix.m24 = event.matrix.m24;
            this.matrix.m31 = event.matrix.m31;
            this.matrix.m32 = event.matrix.m32;
            this.matrix.m33 = event.matrix.m33;
            this.matrix.m34 = event.matrix.m34;
            this.matrix.m41 = event.matrix.m41;
            this.matrix.m42 = event.matrix.m42;
            this.matrix.m43 = event.matrix.m43;
            this.matrix.m44 = event.matrix.m44;
        } else {
            this.x = event.offsetX;
            this.y = event.offsetY;
        }
    }
}
