/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import {
    SBaseCircleEdit,
    SBaseImageEdit,
    SBaseLineEdit,
    SBasePolygonEdit,
    SBaseRectEdit,
    SBaseTextEdit
} from "@persagy-web/edit";
import { Marker, SBaseArrow, SBaseExpainEdit, SBaseArrowPolyEdit, SBaseEquipment, SBasePipe, Legend, Relation } from "./"
import { SItemFactory } from "@persagy-web/big"

/**
 * item 创建工厂
 *
 * @author 韩耀龙 <han_yao_long@163.com>
 */
export class BigEditFactory extends SItemFactory {
    /**
     * 构造函数
     */
    constructor() {
        super()
    }

    /**
     * 创建基础直线
     *
     * @param data  数据
     * @return 线
     */
    createBaseLineEdit(data: Marker): SBaseLineEdit {
        return new SBaseLineEdit(null, data);
    }

    /**
     * 创建基础文本
     *
     * @param data  数据
     * @return 文本
     */
    createBaseTextEdit(data: Marker): SBaseTextEdit {
        return new SBaseTextEdit(null, data);
    }

    /**
     * 创建基础图片
     *
     * @param data  数据
     * @return 图片
     */
    createBaseImageEdit(data: Marker): SBaseImageEdit {
        return new SBaseImageEdit(null, data);
    }

    /**
     * 创建基础矩形
     *
     * @param data  数据
     * @return 矩形
     */
    createBaseRectEdit(data: Marker): SBaseRectEdit {
        return new SBaseRectEdit(null, data);
    }

    /**
     * 创建基础圆
     *
     * @param data  数据
     * @return 圆
     */
    createBaseCircleEdit(data: Marker): SBaseCircleEdit {
        return new SBaseCircleEdit(null, data);
    }

    /**
     * 创建基础注释
     *
     * @param data     数据
     * @return 注释
     */
    createBaseExpainEdit(data: Marker): SBaseExpainEdit {
        return new SBaseExpainEdit(null, data);
    }

    /**
     * 创建基础多边形
     *
     * @param data     数据
     * @return 注释
     */
    createBasePolygonEdit(data: Marker): SBasePolygonEdit {
        return new SBasePolygonEdit(null, data);
    }

    /**
     * 创建基础基础箭头（折线）
     *
     * @param data     数据
     * @return  注释
     */
    createBaseArrow(data: Marker): SBaseArrow {
        return new SBaseArrow(null, data);
    }

    /**
     * 创建基础基础箭头（折线）
     *
     * @param data     数据
     * @return  注释
     */
    createBasePolygonArrowEdit(data: Marker): SBaseArrowPolyEdit {
        return new SBaseArrowPolyEdit(null, data);
    }

    /*
    * 创建基础设备
    *
    * @param data     数据
    * @return 注释
    */
    createBaseEquipment(data: Legend): SBaseEquipment {
        return new SBaseEquipment(null, data);
    }

    /*
     * 创建基础管道
     *
     * @param data     数据
     * @return 注释
     */
    createBasePipe(data: Relation): SBasePipe {
        return new SBasePipe(null, data);
    }
}