/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SMathUtil } from "@persagy-web/graph/lib/utils/SMathUtil";
import { SColor, SLine, SPainter, SPoint } from '@persagy-web/draw/lib';
import { SBaseLineEdit } from "@persagy-web/edit";
import { SItemStatus } from '@persagy-web/big/lib';
import { Marker } from '..';
import { SGraphItem } from '@persagy-web/graph/lib';

/**
 * 箭头编辑(多边形)
 *
 * @author  韩耀龙 <han_yao_long@163.com>
 */
export class SBaseArrowPolyEdit extends SBaseLineEdit {
	/** 多边形数组 */
	pointList: SPoint[] = [];
	/** 绘制时需要旋转的角度 */
	private _ang: number = 0;
	/** 箭头中间粗细 */
	arrowWidth: number = 8;

	/** 是否为单向箭头 是-只绘制末端箭头 */
	_isSingle: boolean = true;
	get isSingle(): boolean {
		return this._isSingle;
	} // Get isSingle
	set isSingle(v: boolean) {
		this._isSingle = v;
		this.update();
	} // Set isSingle

	/**
	 * 构造函数
	 *
	 * @param parent    父级
	 * @param data      坐标集合|第一个点坐标
	 */
	constructor(parent: SGraphItem | null, data: Marker): void {
		super(parent, data);
		this.pointChange();
	}

	/**
	 * 根据两个顶点生成多边形数组
	 */
	pointChange(): void {
		if (this.line.length > 1) {
			const line = new SLine(this.line[0], this.line[1]);
			const dis = SMathUtil.pointDistance(
				this.line[0].x,
				this.line[0].y,
				this.line[1].x,
				this.line[1].y
			);
			if (line.dx != 0) {
				const tempFo = Math.atan(line.dy / line.dx);
				this._ang = line.dx > 0 ? tempFo : tempFo + Math.PI;
			} else {
				this._ang = line.dy > 0 ? Math.PI / 2 : (3 * Math.PI) / 2;
			}

			if (this.isSingle) {
				this.pointList = [
					new SPoint(0, this.arrowWidth / 2),
					new SPoint(dis - this.arrowWidth, this.arrowWidth / 2),
					new SPoint(dis - this.arrowWidth, this.arrowWidth),
					new SPoint(dis, 0),
					new SPoint(dis - this.arrowWidth, -this.arrowWidth),
					new SPoint(dis - this.arrowWidth, -this.arrowWidth / 2),
					new SPoint(0, -this.arrowWidth / 2)
				];
			} else {
				this.pointList = [
					new SPoint(0, 0),
					new SPoint(this.arrowWidth, this.arrowWidth),
					new SPoint(this.arrowWidth, this.arrowWidth / 2),
					new SPoint(dis - this.arrowWidth, this.arrowWidth / 2),
					new SPoint(dis - this.arrowWidth, this.arrowWidth),
					new SPoint(dis, 0),
					new SPoint(dis - this.arrowWidth, -this.arrowWidth),
					new SPoint(dis - this.arrowWidth, -this.arrowWidth / 2),
					new SPoint(this.arrowWidth, -this.arrowWidth / 2),
					new SPoint(this.arrowWidth, -this.arrowWidth)
				];
			}
		}
	}

	/**
	 * 绘制
	 *
	 * @param painter   绘制对象
	 */
	onDraw(painter: SPainter): void {
		if (this.line.length) {
			painter.save();

			const lw = painter.toPx(this.lineWidth);
			painter.pen.lineWidth = lw;
			this.sceneDis = painter.toPx(this.dis);

			painter.translate(this.line[0].x, this.line[0].y);
			painter.rotate((this._ang * 180) / Math.PI);
			painter.pen.color = this.strokeColor;
			painter.brush.color = this.fillColor;

			if (this.selected && this.status == SItemStatus.Normal) {
				painter.pen.lineWidth = 2 * lw;
				painter.shadow.shadowBlur = 10;
				painter.shadow.shadowColor = new SColor(`#00000033`);
				painter.shadow.shadowOffsetX = 5;
				painter.shadow.shadowOffsetY = 5;
			}

			painter.drawPolygon(this.pointList);
			painter.restore();

			if (
				this.status == SItemStatus.Edit ||
				this.status == SItemStatus.Create
			) {
				this.line.forEach((p, i) => {
					painter.brush.color = this.fillColor;
					if (i == this.curIndex) {
						painter.brush.color = this.activeFillColor;
					}

					painter.drawCircle(p.x, p.y, painter.toPx(5));
				});
			}
		}
	}
}