/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SGraphItem, SAnchorItem } from "@persagy-web/graph/lib";
import { Legend } from "./../types/Legend";
import { SBaseIconTextEdit, SBaseTextEdit } from "@persagy-web/edit";
import { SColor } from '@persagy-web/draw/lib';
import { svgTobase64, uuid } from "./../until"
/**
 * 编辑基础设备类
 *
 * @author 韩耀龙 <han_yao_long@163.com>
 */
export class SBaseEquipment extends SBaseIconTextEdit {
    /** 图例节点 id */
    nodeId: string | null = null;
    /** 设备图例 */
    _legendData: Legend | null = null;
    set legendData(val) {
        this._legendData = val;
        this.initData()
    } // set legendData()
    get legendData(): Legend | null {
        return this._legendData
    } // get legendData()

    /** 信息点存储 */
    _infoPointList: any[] = []
    set infoPointList(val) {
        this._infoPointList = val;
    } // set infoPointList()
    get infoPointList(): any[] {
        return this._infoPointList
    } // get infoPointList()

    /** 图得url */
    _url: string = '';
    set url(val) {
        this._url = val;
        this.initUrl()
    } // set infoPointList()
    get url(): string {
        return this._url
    } // get infoPointList()

    //  设备附加数据
    anotherMsg: string = ''

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      数据
     */
    constructor(parent: SGraphItem | null, data: Legend) {
        super(parent);
        this.legendData = data;
    } // Constructor

    /**
     * 设置 legendData 时对 item 做设置
     */
    initData() {
        if (!this.legendData) return;

        this.nodeId = this.legendData?.nodeId
        if (this.legendData.size) {
            this.sWidth = this.legendData.size.width;
            this.sHeight = this.legendData.size.height;
        }

        this.img.connect("onMove", this, this.changeAnchorPoint.bind(this));
        if (this.legendData.anchorList && this.legendData.anchorList.length) {
            this.anchorList = this.legendData.anchorList.map(t => {
                let item = new SAnchorItem(this);
                if (t.anchorId) {
                    item.id = t.anchorId;
                }
                item.moveTo(t.pos.x, t.pos.y);
                return item;
            });
        } else {
            const anchorPoint = [
                { x: this.img.x, y: this.img.y, anchorId: uuid() },
            ];
            this.anchorList = anchorPoint.map(t => {
                let item = new SAnchorItem(this);
                if (t.anchorId) {
                    item.id = t.anchorId;
                }
                item.moveTo(t.x, t.y);
                return item;
            });
        }

        // 存储信息点
        if (this.legendData.properties && this.legendData.properties.infoPointList) {
            const infoPointList = this.legendData.properties.infoPointList;
            if (infoPointList.length) {
                this.infoPointList = infoPointList;
                this.infoPointList.forEach((obj, i) => {
                    this.setTextItem(obj, i)
                })
            } else {
                this.infoPointList = []
            }
        } else {
            this.infoPointList = []
        }

        this.showAnchor = false;
        this.url = this.legendData.style.default.base64Url ? this.legendData.style.default.base64Url : this.legendData.style.default.url
        this.anotherMsg = this.legendData.properties.anotherMsg ? this.legendData.properties.anotherMsg : '';
        this.x = this.legendData.pos.x;
        this.y = this.legendData.pos.y;
        this.moveable = true;
        this.selectable = true;
    }

    initUrl() {
        svgTobase64(this.url).then((res) => {
            this.img.url = res ? res : '';
        }).catch((res) => {
            this.img.url = res;
        })
    }

    /**
     * 设置信息点
     *
     * @params	obj 设置的信息点
     */
    setMsgPoint(val: any) {
        // 根据是否勾选来判断是够增加或删除
        if (val.checked) {
            this._infoPointList.push(val);
            this.setTextItem(val, (this._infoPointList.length - 1))
        } else {
            let deleteItemIndex: number = -1;
            this._infoPointList.forEach((item, index) => {
                if (val.id == item.id) {
                    deleteItemIndex = index;
                }
            });
            this._infoPointList.splice(deleteItemIndex, 1);
            this.setTextItem(val, deleteItemIndex)
        }
    }

    /**
     * 根据选中状态增加 textItem
     *
     * @params obj	textItem 文本信息
     * @params index 文本索引
     */
    setTextItem(val: any, index: number): void {
        if (val.checked) {
            let item = new SBaseTextEdit(this, null);
            item.propertyData = val;
            item.text = val.name;
            if (val.pos) {
                item.moveTo(val.pos.x, val.pos.x)
            } else {
                item.moveTo(
                    this.img.width * 0.5,
                    -(this.font.size * 1.25 * 0.5) + (index) * 10
                );
            }

            item.font.size = val.font ? val.font : 12;
            item.isTransform = false;
            item.showSelect = false;
            item.color = val.color ? new SColor(val.color) : new SColor('#000000');
            item.connect('textSelect', this, this.textSelectChange)
            this.textItemList.push(item)
        } else {
            this.textItemList.splice(index, 1)
            this.scene?.removeItem(this.textItemList[index]);
        }
    }

    /**
     * 返回对象储存的相关数据
     *
     * @return	对象储存的相关数据
     */
    toData(): any {
        if (this.legendData) {
            const list = this.anchorList.map((item) => {
                return {
                    anchorId: item.id,
                    pos: {
                        x: item.x,
                        y: item.y
                    }
                }
            })
            this.legendData.anchorList = list;
            // 反馈大小
            if (this.legendData.size) {
                this.legendData.size.width = this.sWidth;
                this.legendData.size.height = this.sHeight
            } else {
                this.legendData.size = {
                    width: this.sWidth,
                    height: this.sHeight
                }
            }

            // 位置
            this.legendData.pos = {
                x: this.x,
                y: this.y
            };
            // 存储信息点
            const infoPoinList: any[] = []
            this.textItemList.forEach(item => {
                let obj = Object.assign(item.propertyData, {
                    pos: { x: item.x, y: item.y },
                    font: item.font.size,
                    color: item.color.value
                })
                delete obj.currentEquipMsg
                infoPoinList.push(obj)
            });

            this.legendData.properties.infoPointList = infoPoinList;
            this.legendData.properties.anotherMsg = this.anotherMsg;
            this.legendData.style.default.base64Url = "";
            this.legendData.style.default.url = this.url;
            return this.legendData
        }
    }

}
