/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SColor, SPainter, SPoint } from '@persagy-web/draw/lib';
import { SGraphItem } from '@persagy-web/graph/lib';
import { SLineStyle } from '@persagy-web/edit/lib/enums/SLineStyle';

/**
 * 对象关系 虚线 item
 *
 * @author  韩耀龙 <han_yao_long@163.com>
 */
export class SBaseObjectRelation extends SGraphItem {
  /** 线起始点 */
  startPoint: SPoint | null = null;
  /** 结束始点 */
  endPoint: SPoint | null = null;

  /** 线条颜色 */
  private _strokeColor: SColor = SColor.Black;
  get strokeColor(): SColor {
    return this._strokeColor;
  } // Get strokeColor
  set strokeColor(v: SColor) {
    this._strokeColor = v;
    this.update();
  }// Get strokeColor

  /** 线条样式 */
  private _lineStyle: SLineStyle = SLineStyle.Solid;
  get lineStyle(): SLineStyle {
    return this._lineStyle;
  } // Get lineStyle
  set lineStyle(v: SLineStyle) {
    this._lineStyle = v;
    this.update();
  } // Set lineStyle

  /** 线条宽度 */
  private _lineWidth: number = 1;
  get lineWidth(): number {
    return this._lineWidth;
  } //Get lineWidth
  set lineWidth(v: number) {
    this._lineWidth = v;
    this.update();
  } //Set lineWidth

  /**
    * 绘制
    *
    * @param painter   绘制对象
    */
  onDraw(painter: SPainter): void {
    if (this.startPoint && this.endPoint) {
      painter.pen.lineWidth = painter.toPx(this.lineWidth);
      painter.pen.color = this.strokeColor;
      if (this.lineStyle == SLineStyle.Dashed) {
        painter.pen.lineDash = [
          painter.toPx(this.lineWidth * 3),
          painter.toPx(this.lineWidth * 7)
        ];
      } else if (this.lineStyle == SLineStyle.Dotted) {
        painter.pen.lineDash = [
          painter.toPx(2 * this.lineWidth),
          painter.toPx(2 * this.lineWidth)
        ];
      }

      painter.drawLine(this.startPoint, this.endPoint);
    }
  }
}