/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { Column } from "../types/floor/Column";
import { SColumnItem } from "../items/floor/SColumnItem";
import { SWallItem } from "../items/floor/SWallItem";
import { Wall } from "../types/floor/Wall";
import { SVirtualWallItem } from "../items/floor/SVirtualWallItem";
import { VirtualWall } from "../types/floor/VirtualWall";
import { SSpaceItem } from "../items/floor/SSpaceItem";
import { Space } from "../types/floor/Space";
import { Door } from "../types/floor/Door";
import { SDoorItem } from "../items/floor/SDoorItem";
import { SWindowItem } from "../items/floor/SWindowItem";
import { Casement } from "../types/floor/Casement";
import { SZoneItem } from "../items/floor/ZoneItem";
import { Zone } from "../types/floor/Zone";

/**
 * item 创建工厂
 *
 * @author 郝建龙 <haojianlong@sagacloud.cn>
 */
export class SItemFactory {
    /**
     * 构造函数
     */
    constructor() {}

    /**
     * 创建柱子 item
     *
     * @param data    柱子数据
     * @return 柱子 item
     */
    createColumn(data: Column): SColumnItem {
        return new SColumnItem(null, data);
    }

    /**
     * 创建墙 item
     *
     * @param data    墙数据
     * @return 墙 item
     */
    createWall(data: Wall): SWallItem {
        return new SWallItem(null, data);
    }

    /**
     * 创建虚拟墙 item
     *
     * @param data    虚拟墙数据
     * @return 虚拟墙 item
     */
    createVirtualWall(data: VirtualWall): SVirtualWallItem {
        return new SVirtualWallItem(null, data);
    }

    /**
     * 创建空间 item
     *
     * @param data    空间数据
     * @return 空间 item
     */
    createSpace(data: Space): SSpaceItem {
        return new SSpaceItem(null, data);
    }

    /**
     * 创建门 item
     *
     * @param data    门数据
     * @return 门item
     */
    createDoor(data: Door): SDoorItem {
        return new SDoorItem(null, data);
    }

    /**
     * 创建窗 item
     *
     * @param data    窗户数据
     * @return 窗户 item
     */
    createWindow(data: Casement): SWindowItem {
        return new SWindowItem(null, data);
    }

    /**
     * 创建业务空间 item
     *
     * @param data    业务空间数据
     * @return 业务空间 item
     */
    createZone(data: Zone): SZoneItem {
        return new SZoneItem(null, data);
    }
}
