/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SPainter, SArrowStyleType } from "@persagy-web/draw";
import { SPolylineItem } from "..";
import { SLineStyle } from "@persagy-web/graph";

/**
 * 折线箭头 item
 *
 * @author 郝建龙 <haojianlong@sagacloud.cn>
 */
export class SArrowItem extends SPolylineItem {
    /** 起始点样式 */
    _begin = SArrowStyleType.None;
    get begin(): SArrowStyleType {
        return this._begin;
    }
    set begin(v: SArrowStyleType) {
        this._begin = v;
        this.update();
    }

    /** 结束点样式 */
    _end = SArrowStyleType.None;
    get end(): SArrowStyleType {
        return this._end;
    }
    set end(v: SArrowStyleType) {
        this._end = v;
        this.update();
    }

    /**
     * 绘制基本图形
     *
     * @param painter   绘制对象
     */
    drawBaseLine(painter: SPainter): void {
        // 绘制基本图形，线样式是虚线时
        if (this.lineStyle == SLineStyle.Dashed) {
            painter.pen.lineDash = [
                painter.toPx(this.lineWidth * 3),
                painter.toPx(this.lineWidth * 7)
            ];
        } else if (this.lineStyle == SLineStyle.Dotted) {
            // 绘制基本图形，线样式是点线时
            painter.pen.lineDash = [
                painter.toPx(this.lineWidth),
                painter.toPx(this.lineWidth)
            ];
        }

        painter.pen.color = this.strokeColor;
        painter.drawArrowLine(this.pointList[0], this.pointList[1], {
            begin: this.begin,
            end: SArrowStyleType.None
        });
        painter.drawArrowLine(
            this.pointList[this.pointList.length - 2],
            this.pointList[this.pointList.length - 1],
            {
                begin: SArrowStyleType.None,
                end: this.end
            }
        );
        painter.drawPolyline(this.pointList);
    }
}
