/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SGraphItem } from "@persagy-web/graph";
import { SPainter, SPoint, SRect } from "@persagy-web/draw";
import { SMouseEvent } from "@persagy-web/base";
import { ItemColor, ItemOrder } from "..";

/**
 * 矩形选择 item
 *
 * @author  郝建龙 <haojianlong@sagacloud.cn>
 */
export class SRectSelectItem extends SGraphItem {
    /** 起点 */
    startPoint: SPoint = new SPoint();
    /** 终点 */
    endPoint: SPoint = new SPoint();

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param point     起点数据
     */
    constructor(parent: SGraphItem | null, point: SPoint) {
        super(parent);
        this.startPoint = point;
        this.endPoint = new SPoint(point.x, point.y);
        this.update();
        this.zOrder = ItemOrder.rectSelectOrder;
    }

    /**
     * Item 对象边界区域
     *
     * @return 边界区域
     */
    boundingRect(): SRect {
        return new SRect(this.startPoint, this.endPoint);
    }

    /**
     * 鼠标移动事件
     *
     * @param event     事件参数
     * @return 是否处理
     */
    onMouseMove(event: SMouseEvent): boolean {
        this.endPoint.x = event.x;
        this.endPoint.y = event.y;
        this.update();
        return true;
    }

    /**
     * Item 绘制操作
     *
     * @param painter   绘制对象
     */
    onDraw(painter: SPainter): void {
        painter.pen.lineWidth = painter.toPx(2);
        painter.pen.color = ItemColor.rectSelectOutColor;
        painter.brush.color = ItemColor.rectSelectInColor;
        painter.drawRect(this.startPoint, this.endPoint);
    }
}
