/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SPainter, SPath, SPoint } from "@persagy-web/draw";
import { Zone } from "../../types/floor/Zone";
import { ItemColor } from "../..";
import { SGraphItem } from "@persagy-web/graph";

/**
 * 建筑轮廓 item
 *
 * @author  郝建龙 <haojianlong@sagacloud.cn>
 */
export class SBoardItem extends SGraphItem {
    /** 空间所有数据 */
    data: Zone;
    /** 空间轮廓线坐标 list */
    readonly pointArr: SPoint[][][] = [];
    /** path 数组 */
    private pathList: SPath[] = [];

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      建筑轮廓数据
     */
    constructor(parent: SGraphItem | null, data: Zone) {
        super(parent);
        this.data = data;
        this.zOrder = 0;
        // 判断是否有轮廓数据
        if (
            this.data.OutLine.length &&
            this.data.OutLine[0] &&
            this.data.OutLine[0].length
        ) {
            let tempArr = this.data.OutLine;
            this.pointArr = tempArr.map((t): SPoint[][] => {
                let sPath = new SPath();
                let tempArr = t.map((it): SPoint[] => {
                    let array = it.map(
                        (item): SPoint => {
                            let x = item.X,
                                y = -item.Y;
                            return new SPoint(x, y);
                        }
                    );
                    sPath.polygon(array);
                    return array;
                });
                this.pathList.push(sPath);
                return tempArr;
            });
        }
    }

    /**
     * 判断点是否在区域内
     *
     * @param x     x 坐标
     * @param y     y 坐标
     * @return 是否在区域内
     */
    contains(x: number, y: number): boolean {
        return false;
    }

    /**
     * Item 绘制操作
     *
     * @param painter   绘制对象
     */
    onDraw(painter: SPainter): void {
        painter.pen.lineWidth = painter.toPx(1);
        painter.brush.color = ItemColor.spaceColor;
        painter.pen.color = ItemColor.spaceBorderColor;
        this.pathList.forEach((t): void => {
            painter.drawPath(t);
        });
    }
}
