/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SColor, SPainter, STextAlign } from "@persagy-web/draw";
import { Space } from "../../types/floor/Space";
import { ItemOrder, ItemColor } from "../..";
import { SGraphItem, SGraphAreaGroupItem } from "@persagy-web/graph";
import { SMouseEvent } from "@persagy-web/base";

/**
 * 模型空间 item
 *
 * @author  郝建龙 <haojianlong@sagacloud.cn>
 */
export class SSpaceItem extends SGraphAreaGroupItem {
    /** 空间所有数据 */
    data: Space;
    /** 高亮状态 */
    private _highLightFlag: boolean = false;
    get highLightFlag(): boolean {
        return this._highLightFlag;
    }

    set highLightFlag(value: boolean) {
        this._highLightFlag = value;
        this.update();
    }

    /** 是否显示名字 */
    private _showBaseName: boolean = false;
    get showBaseName(): boolean {
        return this._showBaseName;
    }

    set showBaseName(value: boolean) {
        this._showBaseName = value;
        this.update();
    }

    /** 是否名字大小 */
    private _nameSize: number = 10;
    get nameSize(): number {
        return this._nameSize;
    }

    set nameSize(value: number) {
        this._nameSize = value;
        this.update();
    }

    /** 名字是否缩放 */
    private _nameTransform: boolean = false;
    get nameTransform(): boolean {
        return this._nameTransform;
    }

    set nameTransform(value: boolean) {
        this._nameTransform = value;
        this.update();
    }

    /** 名字颜色 */
    private _nameColor: string = "#000000";
    get nameColor(): string {
        return this._nameColor;
    }

    set nameColor(value: string) {
        this._nameColor = value;
        this.update();
    }

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      空间数据
     */
    constructor(parent: SGraphItem | null, data: Space) {
        super(parent, {
            outline: [
                data.OutLine.map((t): any =>
                    t.map((it): any => ({ x: it.X, y: -it.Y }))
                )
            ],
            style: {
                default: {
                    stroke: "#2b2b2b",
                    fill: "#f1fffd80",
                    lineWidth: 1,
                    effect: {}
                },
                selected: {
                    stroke: "#2b2b2b",
                    fill: "#1abc9c",
                    lineWidth: 1,
                    effect: {}
                },
                disabled: {
                    stroke: "#2b2b2b",
                    fill: "#afafaf",
                    lineWidth: 1,
                    effect: {}
                },
                highlight: {
                    stroke: "#2b2b2b",
                    fill: "#fbb029",
                    lineWidth: 1,
                    effect: {}
                }
            }
        });

        this.data = data;
        this.zOrder = ItemOrder.spaceOrder;
        this.name = data.Name || "";
    }

    /**
     * 绘制前设置绘制样式
     *
     * @return 当前状态
     */
    setStyle(): string {
        let status = "default";
        // 如果存在样式
        if (this.style) {
            // 如果处于高亮状态
            if (this.highLightFlag) {
                status = "highlight";
            } else if (this.enabled) {
                // 是否可用,true 可用
                if (this.selected) {
                    // 是否选中
                    status = "selected";
                }
            } else {
                status = "disabled";
            }
        }
        return status;
    }

    /**
     * 鼠标按下事件
     *
     * @param event   保存事件参数
     * @return 是否处理事件
     */
    onMouseDown(event: SMouseEvent): boolean {
        this.$emit("onMouseDown", event);
        return true;
    }

    /**
     * 鼠标抬起事件
     *
     * @param event   保存事件参数
     * @return 是否处理事件
     */
    onMouseUp(event: SMouseEvent): boolean {
        this.$emit("onMouseUp", event);
        return true;
    }

    /**
     * 鼠标移入事件
     *
     * @param event   保存事件参数
     * @return 是否处理事件
     */
    onMouseEnter(event: SMouseEvent): boolean {
        this.$emit("onMouseEnter", event);
        return false;
    }

    /**
     * 鼠标移出事件
     *
     * @param event   保存事件参数
     * @return 是否处理事件
     */
    onMouseLeave(event: SMouseEvent): boolean {
        this.$emit("onMouseLeave", event);
        return false;
    }

    /**
     * Item 绘制操作
     *
     * @param painter   绘制对象
     */
    onDraw(painter: SPainter): void {
        super.onDraw(painter);
        // 如果显示名字
        if (this.showBaseName) {
            if (this.name && this.name != "null") {
                painter.brush.color = new SColor(this.nameColor);
                if (this.nameTransform) {
                    painter.font.size = this.nameSize;
                } else {
                    painter.font.size = painter.toPx(this.nameSize);
                }
                painter.font.textAlign = STextAlign.Center;
                painter.drawText(
                    this.name,
                    this.data.Location.Points[0].X,
                    -this.data.Location.Points[0].Y
                );
            }
        }
    }
}
