/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SPainter, SPoint, SRect } from "@persagy-web/draw";
import { VirtualWall } from "../../types/floor/VirtualWall";
import { ItemOrder, ItemColor } from "../..";
import { SGraphItem } from "@persagy-web/graph";

/**
 * 虚拟墙 item
 *
 * @author  郝建龙 <haojianlong@sagacloud.cn>
 */
export class SVirtualWallItem extends SGraphItem {
    /** 虚拟墙数据 */
    data: VirtualWall;
    /** X 坐标最小值 */
    private minX = Number.MAX_SAFE_INTEGER;
    /** X 坐标最大值 */
    private maxX = Number.MIN_SAFE_INTEGER;
    /** Y 坐标最小值 */
    private minY = Number.MAX_SAFE_INTEGER;
    /** Y 坐标最大值 */
    private maxY = Number.MIN_SAFE_INTEGER;
    /** 虚拟墙轮廓线坐标 list */
    private readonly pointArr: SPoint[][] = [];

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      虚拟墙数据
     */
    constructor(parent: SGraphItem | null, data: VirtualWall) {
        super(parent);
        this.data = data;
        this.zOrder = ItemOrder.virtualWallOrder;
        let tempArr = this.data.OutLine;
        // 如果数据存在
        if (tempArr && tempArr.length) {
            this.minX = tempArr[0][0].X;
            this.maxX = this.minX;
            this.minY = -tempArr[0][0].Y;
            this.maxY = this.minY;
            this.pointArr = tempArr.map((t): SPoint[] => {
                return t.map(
                    (it): SPoint => {
                        let x = it.X,
                            y = -it.Y;

                        // 如果数据 x 坐标小于 x 坐标最小值
                        if (x < this.minX) {
                            this.minX = x;
                        }

                        // 如果数据 y 坐标小于 y 坐标最小值
                        if (y < this.minY) {
                            this.minY = y;
                        }

                        // 如果数据 x 坐标大于 x 坐标最小值
                        if (x > this.maxX) {
                            this.maxX = x;
                        }

                        // 如果数据 y 坐标大于 y 坐标最小值
                        if (y > this.maxY) {
                            this.maxY = y;
                        }

                        return new SPoint(x, y);
                    }
                );
            });
        }
    }

    /**
     * Item 对象边界区域
     *
     * @return 边界矩形
     */
    boundingRect(): SRect {
        return new SRect(
            this.minX,
            this.minY,
            this.maxX - this.minX,
            this.maxY - this.minY
        );
    }

    /**
     * Item 绘制操作
     *
     * @param painter   绘制对象
     */
    onDraw(painter: SPainter): void {
        painter.pen.lineWidth = painter.toPx(1);
        painter.pen.color = ItemColor.virtualWallColor;
        painter.pen.lineDash = [20, 20];
        this.pointArr.forEach((t): void => {
            painter.drawPolyline(t);
        });
    }
}
