/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SColor, SPainter, SPoint, SRect } from "@persagy-web/draw";
import { SMouseEvent } from "@persagy-web/base";
import { Zone } from "../../types/floor/Zone";
import { ItemColor, ItemOrder, Transparency } from "../..";
import { Point, SGraphAreaGroupItem, SGraphItem } from "@persagy-web/graph";

/**
 * 业务空间 item
 *
 * @author  郝建龙 <haojianlong@sagacloud.cn>
 */
export class SZoneItem extends SGraphAreaGroupItem {
    /** 空间所有数据 */
    data: Zone;
    /** 点击位置坐标 */
    private clickPoint: SPoint | undefined;
    /** 高亮状态 */
    private _highLightFlag: boolean = false;
    get highLightFlag(): boolean {
        return this._highLightFlag;
    }

    set highLightFlag(value: boolean) {
        this._highLightFlag = value;
        this.update();
    }

    /** 透明度 */
    _transparency: number = 20;
    get transparency(): number {
        return this._transparency;
    }

    set transparency(value: number) {
        this._transparency = value;
        this.update();
    }

    /** 受影响状态 */
    _isInfected: boolean = false;
    get isInfected(): boolean {
        return this._isInfected;
    }

    set isInfected(value: boolean) {
        this._isInfected = value;
        this.update();
    }

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      空间数据
     */
    constructor(parent: SGraphItem | null, data: Zone) {
        super(parent, {
            outline: data.OutLine.map((t): Point[][] => {
                return t.map((it): Point[] => {
                    return it.map(
                        (item): Point => {
                            return { x: item.X, y: -item.Y };
                        }
                    );
                });
            }),
            style: {
                default: {
                    stroke: "#00000000",
                    fill: "#ffee2280",
                    lineWidth: 1,
                    effect: {}
                },
                selected: {
                    stroke: "#00000000",
                    fill: "#1abc9c",
                    lineWidth: 1,
                    effect: {}
                },
                highlight: {
                    stroke: "#00000000",
                    fill: data.Color || "#fbb029",
                    lineWidth: 1,
                    effect: {}
                },
                unselect: {
                    stroke: "#00000000",
                    fill: "#cecece",
                    lineWidth: 1,
                    effect: {}
                },
                infected: {
                    stroke: "#00000000",
                    fill: "#ff0000",
                    lineWidth: 1,
                    effect: {}
                }
            }
        });

        this.data = data;
        this.zOrder = ItemOrder.zoneOrder;
        this.highLightFlag = data.HighLightFlag || false;
        this.transparency = data.Transparency || 20;
        this.isInfected = data.Infected || false;
    }

    /**
     * 鼠标单击事件
     *
     * @param event     事件参数
     * @return 是否处理事件
     */
    onMouseDown(event: SMouseEvent): boolean {
        // 是否可选中
        if (this.selectable) {
            this.selected = !this.selected;
            this.clickPoint = new SPoint(event.x, event.y);
        }
        this.$emit("click", event);
        return true;
    }

    /**
     * 鼠标抬起事件
     *
     * @param event     事件参数
     * @return 是否处理事件
     */
    onMouseUp(event: SMouseEvent): boolean {
        return false;
    }

    /**
     * 绘制前设置绘制样式
     *
     * @return 当前状态
     */
    setStyle(): string {
        let status = "default";
        // 是否存在样式
        if (this.style) {
            // 是否可选中
            if (!this.selectable) {
                status = "unselect";
            } else {
                // 是否选中
                if (this.selected) {
                    status = "selected";
                } else if (this.highLightFlag) {
                    // 高亮状态
                    status = "highlight";
                } else if (this.isInfected) {
                    // 受影响
                    status = "infected";
                } else {
                    status = "default";
                    if (this.data.Color) {
                        // 数据是否存在 color
                        this.style.default.fill = `${this.data.Color}${
                            Transparency[this.transparency]
                        }`;
                    }
                }
            }
        }
        return status;
    }

    /**
     * Item 绘制操作
     *
     * @param painter   绘制对象
     */
    onDraw(painter: SPainter): void {
        super.onDraw(painter);
        painter.brush.color = SColor.Black;
        painter.font.size = painter.toPx(10);
        // 是否点击位置坐标
        if (this.clickPoint) {
            painter.drawText(
                this.data.RoomLocalName,
                this.clickPoint.x,
                this.clickPoint.y
            );
        }
    }
}
