/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SParser } from "./SParser";
import { FloorData } from "../types/FloorData";
import { Wall } from "../types/floor/Wall";
import { Column } from "../types/floor/Column";
import { Casement } from "../types/floor/Casement";
import { VirtualWall } from "../types/floor/VirtualWall";
import { Door } from "../types/floor/Door";
import { Space } from "../types/floor/Space";
import { SWallItem } from "../items/floor/SWallItem";
import { SColumnItem } from "../items/floor/SColumnItem";
import { SDoorItem } from "../items/floor/SDoorItem";
import { SWindowItem } from "../items/floor/SWindowItem";
import { SVirtualWallItem } from "../items/floor/SVirtualWallItem";
import { SSpaceItem } from "../items/floor/SSpaceItem";

/**
 * 楼层信息解析器
 *
 * @author 郝建龙 <haojianlong@sagacloud.cn>
 */
export class SFloorParser extends SParser {
    /** 墙 list */
    wallList: SWallItem[] = [];
    /** 柱子 list */
    columnList: SColumnItem[] = [];
    /** 门 list */
    doorList: SDoorItem[] = [];
    /** 窗 list */
    casementList: SWindowItem[] = [];
    /** 虚拟墙 list */
    virtualWallList: SVirtualWallItem[] = [];
    /** 空间 list */
    spaceList: SSpaceItem[] = [];

    /**
     * 解析数据
     *
     * @param data  楼层数据
     */
    parseData(data: FloorData): void {
        // 墙数据
        if (data.Walls) {
            data.Walls.forEach((t: Wall): void => {
                this.addWall(t);
            });
        }

        if (data.Columns) {
            data.Columns.forEach((t: Column): void => {
                this.addColumn(t);
            });
        }

        if (data.Windows) {
            data.Windows.forEach((t: Casement): void => {
                this.addCasement(t);
            });
        }

        if (data.VirtualWalls) {
            data.VirtualWalls.forEach((t: VirtualWall): void => {
                this.addVirtualWall(t);
            });
        }

        if (data.Doors) {
            data.Doors.forEach((t: Door): void => {
                this.addDoor(t);
            });
        }

        if (data.Spaces) {
            data.Spaces.forEach((t: Space): void => {
                this.addSpace(t);
            });
        }
    }

    /**
     * 添加墙至场景中
     *
     * @param t     墙数据
     */
    private addWall(t: Wall): void {
        let item = this.factory.createWall(t);
        this.wallList.push(item);
    }

    /**
     * 添加柱子至场景中
     *
     * @param t   柱子数据
     */
    private addColumn(t: Column): void {
        let item = this.factory.createColumn(t);
        this.columnList.push(item);
    }

    /**
     * 添加窗户至场景中
     *
     * @param t     窗户数据
     */
    private addCasement(t: Casement): void {
        let item = this.factory.createWindow(t);
        this.casementList.push(item);
    }

    /**
     * 添加虚拟墙至场景中
     *
     * @param t     虚拟墙数据
     */
    private addVirtualWall(t: VirtualWall): void {
        let item = this.factory.createVirtualWall(t);
        this.virtualWallList.push(item);
    }

    /**
     * 添加门至场景中
     *
     * @param t     门数据
     */
    private addDoor(t: Door): void {
        let item = this.factory.createDoor(t);
        this.doorList.push(item);
    }

    /**
     * 添加空间至场景中
     *
     * @param t     空间数据
     */
    private addSpace(t: Space): void {
        let item = this.factory.createSpace(t);
        this.spaceList.push(item);
    }
}
