/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SColor, SLineCapStyle, SLineJoinStyle } from "./";

/**
 * 画笔
 *
 * @author 庞利祥 <sybotan@126.com>
 */
export class SPen {
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 属性定义
    /** 画笔颜色 */
    color = SColor.Black;

    /** 线宽 */
    lineWidth = 1;

    /** 线段端点样式 */
    lineCapStyle = SLineCapStyle.Butt;

    /** 线段与线段间接合处的样式 */
    lineJoinStyle = SLineJoinStyle.Miter;

    /** 限制当两条线相交时交接处最大长度 (默认值是10.0) */
    miterLimit = 10;

    /** 虚线样式 */
    lineDash: number[] | null = null;

    /** 虚线样式的起始偏移量 */
    dashOffset = 0;

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 构造函数
    /**
     * 构造函数
     */
    constructor();

    /**
     * 构造函数
     *
     * @param pen   画笔
     */
    constructor(pen: SPen);

    /**
     * 构造函数
     *
     * @param color       颜色
     * @param lineWidth   线宽
     */
    constructor(color: SColor, lineWidth?: number);

    /**
     * 构造函数
     *
     * @param color       颜色 | 画笔
     * @param lineWidth   线宽
     */
    constructor(color?: SColor | SPen, lineWidth?: number) {
        if (color == undefined) {
            return;
        }
        if (color instanceof SPen) {
            this.color = new SColor(color.color);
            this.lineWidth = color.lineWidth;
            this.lineCapStyle = color.lineCapStyle;
            this.lineJoinStyle = color.lineJoinStyle;
            this.miterLimit = color.miterLimit;
            this.dashOffset = color.dashOffset;
            this.lineDash =
                color.lineDash != null ? color.lineDash.slice() : null;
        } else {
            this.color = new SColor(color);
            this.lineWidth = lineWidth != undefined ? lineWidth : 1;
        }
    }
}
