/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SGradient, SPoint } from "./";

/**
 * 放射性渐变
 *
 * @author 庞利祥 <sybotan@126.com>
 */
export class SRadialGradient extends SGradient {
    /** 开始圆形半径 */
    r1: number;
    /** 结束圆形半径 */
    r2: number;

    /**
     * 构造函数
     *
     * @param start   开始圆形坐标
     * @param r1      开始圆形半径
     * @param end     结束圆形坐标
     * @param r2      结束圆形半径
     */
    constructor(start: SPoint, r1: number, end: SPoint, r2: number);

    /**
     * 构造函数
     *
     * @param x1      开始圆形 X 坐标
     * @param y1      开始圆形 Y 坐标
     * @param r1      开始圆形半径
     * @param x2      结束圆形 X 坐标
     * @param y2      结束圆形 Y 坐标
     * @param r2      结束圆形半径
     */
    constructor(
        x1: number,
        y1: number,
        r1: number,
        x2: number,
        y2: number,
        r2: number
    );

    /**
     * 构造函数（重载实现）
     *
     * @param x1      开始圆形 X 坐标 | 开始圆形坐标
     * @param y1      开始圆形 Y 坐标 | 开始圆形半径
     * @param r1      开始圆形半径 | 结束圆形坐标
     * @param x2      结束圆形 X 坐标 | 结束圆形半径
     * @param y2      结束圆形 Y 坐标
     * @param r2      结束圆形半径
     */
    constructor(
        x1: number | SPoint,
        y1: number,
        r1: number | SPoint,
        x2: number,
        y2?: number,
        r2?: number
    ) {
        super();
        if (x1 instanceof SPoint && r1 instanceof SPoint) {
            this.start = new SPoint(x1);
            this.end = new SPoint(r1);
            this.r1 = y1;
            this.r2 = x2;
        } else {
            this.start = new SPoint(x1 as number, y1 as number);
            this.end = new SPoint(x2 as number, y2 as number);
            this.r1 = r1 as number;
            this.r2 = r2 as number;
        }
    }

    /**
     * 转为字符串保存
     *
     */
    value(): string {
        let str = `SRadialGradient{`;
        str += `${this.x1},${this.y1},${this.r1};`;
        str += `${this.x2},${this.y2},${this.r2};`;
        this.stopList.forEach((t): void => {
            str += `${t.pos},${t.color.value};`;
        });
        return `${str}}`;
    }
}
