/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SBrush, SFont, SPath, SPen } from "..";
import { SCompositeType } from "..";
import { SShadow } from "../SShadow";
import { SMatrix } from "@persagy-web/base";

/**
 * 绘制状态
 *
 * @author 庞利祥 <sybotan@126.com>
 */
export class SPaintState {
    /** 变换矩阵 */
    matrix = new SMatrix();

    /** 画笔 */
    private _pen = new SPen();
    get pen(): SPen {
        return this._pen;
    }
    set pen(value: SPen) {
        this._pen = value;
    }

    /** 画刷 */
    private _brush = new SBrush();
    get brush(): SBrush {
        return this._brush;
    }
    set brush(value: SBrush) {
        this._brush = value;
    }

    /** 字体 */
    private _font = new SFont();
    get font(): SFont {
        return this._font;
    }
    set font(value: SFont) {
        this._font = value;
    }

    /** 融合 */
    _composite = SCompositeType.SourceOver;

    /** 阴影效果 */
    private _shadow: SShadow = new SShadow();
    get shadow(): SShadow {
        return this._shadow;
    }
    set shadow(v: SShadow) {
        this._shadow = v;
    }

    /** 裁剪 */
    private _clip: SPath | undefined;
    get clip(): SPath | undefined {
        return this._clip;
    }
    set clip(v: SPath | undefined) {
        if (v) {
            this._clip = new SPath();
            this._clip.addPath(v);
        } else {
            this._clip = v;
        }
    }

    /**
     * 构造函数
     *
     * @param state     绘制状态
     */
    constructor(state?: SPaintState) {
        if (state != undefined) {
            this.pen = new SPen(state.pen);
            this.brush = new SBrush(state.brush);
            this.font = new SFont(state.font);
            this._composite = state._composite;
            this.shadow = new SShadow(state.shadow);
            this.clip = state.clip;
            let m = new SMatrix();
            m.m11 = state.matrix.m11;
            m.m12 = state.matrix.m12;
            m.m13 = state.matrix.m13;
            m.m14 = state.matrix.m14;
            m.m21 = state.matrix.m21;
            m.m22 = state.matrix.m22;
            m.m23 = state.matrix.m23;
            m.m24 = state.matrix.m24;
            m.m31 = state.matrix.m31;
            m.m32 = state.matrix.m32;
            m.m33 = state.matrix.m33;
            m.m34 = state.matrix.m34;
            m.m41 = state.matrix.m41;
            m.m42 = state.matrix.m42;
            m.m43 = state.matrix.m43;
            m.m44 = state.matrix.m44;
            this.matrix = m;
        }
    }
}
