/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SMatrix } from "@persagy-web/base";

/**
 * 点数据类型定义
 *
 * @author 庞利祥 <sybotan@126.com>
 */
export class SPoint {
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 属性定义
    /** X轴坐标 */
    x: number;
    /** Y轴坐标 */
    y: number;

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 构造函数
    /**
     * 构造函数
     */
    constructor();

    /**
     * 构造函数
     *
     * @param p     点
     */
    constructor(p: SPoint);

    /**
     * 构造函数
     *
     * @param x     X 轴坐标
     * @param y     Y 轴坐标
     */
    constructor(x: number, y: number);

    /**
     * 构造函数(重载实现)
     *
     * @param x     X 轴坐标 | 点 | undefined
     * @param y     Y 轴坐标
     */
    constructor(x?: number | SPoint, y?: number) {
        if (x == undefined) {
            this.x = 0;
            this.y = 0;
        } else if (x instanceof SPoint) {
            this.x = x.x;
            this.y = x.y;
        } else {
            this.x = x as number;
            this.y = y as number;
        }
    }

    /**
     * 设置点
     *
     * @param pos   点
     */
    setPoint(pos: SPoint): void;

    /**
     * 设置点
     *
     * @param x     X 轴坐标
     * @param y     Y 轴坐标
     */
    setPoint(x: number, y: number): void;

    /**
     * 设置点(重载实现)
     *
     * @param x     X 轴坐标 | 点
     * @param y     Y 轴坐标
     */
    setPoint(x: number | SPoint, y?: number): void {
        if (x instanceof SPoint) {
            this.x = x.x;
            this.y = x.y;
        } else {
            this.x = x as number;
            this.y = y as number;
        }
    }

    /**
     * 布线长度
     *
     * @return 返回 |X| + |Y|
     */
    manhattanLength(): number {
        // noinspection JSSuspiciousNameCombination
        return Math.abs(this.x) + Math.abs(this.y);
    }

    /**
     * 返回点矩阵变换后的值
     *
     * @return 变换后的点
     */
    matrixTransform(mat: SMatrix): SPoint {
        const p = new SPoint();
        p.x = this.x * mat.a + this.y * mat.b + mat.e;
        p.y = this.x * mat.c + this.y * mat.d + mat.f;
        return p;
    }

    /**
     * 平移点
     *
     * @param dx    X 轴位移
     * @param dy    Y 轴位移
     */
    translate(dx: number, dy: number): void {
        this.x += dx;
        this.y += dy;
    }
}
