/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SAspectRatioMode } from "..";

/**
 * 大小数据类型定义
 *
 * @author 庞利祥 <sybotan@126.com>
 */
export class SSize {
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 属性定义
    /** 宽度 */
    width: number;
    /** 高度 */
    height: number;

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 构造函数
    /**
     * 构造函数
     */
    constructor();

    /**
     * 构造函数
     *
     * @param size    大小
     */
    constructor(size: SSize);

    /**
     * 构造函数(重载实现)
     *
     * @param width   宽度
     * @param height  高度
     */
    constructor(width: number, height: number);

    /**
     * 构造函数(重载实现)
     *
     * @param width   宽度
     * @param height  高度
     */
    constructor(width?: number | SSize, height?: number) {
        if (width == undefined) {
            this.width = 0;
            this.height = 0;
        } else if (width instanceof SSize) {
            this.width = width.width;
            this.height = width.height;
        } else {
            this.width = width as number;
            this.height = height as number;
        }
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 方法
    /**
     * 是否为空
     *
     * @return width 或 height 小于等于 0，返回 true，否则返回 false。
     *
     * @see isNull(), isValid()
     */
    isEmpty(): boolean {
        return this.width <= 0 || this.height <= 0;
    }

    /**
     * 是否为 Null
     *
     * @return width 与 height 都等于 0，返回 true，否则返回 false。
     *
     * @see isEmpty(), isValid()
     */
    isNull(): boolean {
        return this.width == 0 && this.height == 0;
    }

    /**
     * 是否有效
     *
     * @return width与 height 都大于 0，返回 true，否则返回 false。
     *
     * @see isEmpty(), isNull()
     */
    isValid(): boolean {
        return this.width > 0 && this.height > 0;
    }

    /**
     * 缩放
     *
     * @param width       宽度
     * @param height      高度
     * @param mode        缩放模式
     */
    scale(
        width: number,
        height: number,
        mode: SAspectRatioMode = SAspectRatioMode.IgnoreAspectRatio
    ): void {
        let s = this.scaled(width, height, mode);
        this.width = s.width;
        this.height = s.height;
    }

    /**
     * 生成缩放大小
     *
     * @param width       宽度
     * @param height      高度
     * @param mode        缩放模式
     * @return 缩放大小
     */
    scaled(
        width: number,
        height: number,
        mode: SAspectRatioMode = SAspectRatioMode.IgnoreAspectRatio
    ): SSize {
        let w = 0;
        let h = 0;
        if (mode == SAspectRatioMode.KeepAspectRatio) {
            let s = Math.min(width / this.width, height / this.height);
            w = this.width * s;
            h = this.height * s;
        } else if (mode == SAspectRatioMode.KeepAspectRatioByExpanding) {
            let s = Math.max(width / this.width, height / this.height);
            w = this.width * s;
            h = this.height * s;
        } else {
            // 自由模式
            w = width;
            h = height;
        }
        return new SSize(w, h);
    }

    /**
     * 互换 width 与 height
     *
     * @see transposed()
     */
    transpose(): void {
        let size = this.transposed();
        this.width = size.width;
        this.height = size.height;
    }

    /**
     * 生成互换 width 与 height 的 SSize
     *
     * @return 返回互换 width 与 height 的 SSize。
     *
     * @see transpose()
     */
    transposed(): SSize {
        return new SSize(this.height, this.width);
    }
}
