/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SPoint } from "..";

/**
 * 多边形工具
 *
 * @author 庞利祥 <sybotan@126.com>
 */
export class SPolygonUtil {
    /**
     * 判断点是否在多边形内
     *
     * @param x           点的 x 坐标
     * @param y           点的 y 坐标
     * @param pointList   多边形顶点
     */
    static pointIn(x: number, y: number, pointList: SPoint[]): boolean {
        let crossNum = 0;
        for (let i = 0; i < pointList.length; i++) {
            let p1 = pointList[i];
            let p2 = pointList[(i + 1) % pointList.length];

            // p1p2 与 y=p0.y平行
            if (p1.y == p2.y) {
                continue;
            }

            // 交点在p1p2延长线上
            if (y < Math.min(p1.y, p2.y) || y >= Math.max(p1.y, p2.y)) {
                continue;
            }

            // 求交点的 x 坐标
            let x0 = ((y - p1.y) * (p2.x - p1.x)) / (p2.y - p1.y) + p1.x;

            if (x0 > x) {
                // 只统计单边交点
                crossNum++;
            }
        }

        // 单边交点为奇数，点在多边形之内
        return crossNum % 2 == 1;
    }

    /**
     * 判断多边型顶点的时针方向（计算基于数学坐标系统，在绘图坐标系由于y轴方向相反结果也要取反）
     *
     * @param pointList   多边形顶点
     * @return 为负，顺时针；为正，逆时针；0，点在直线上。
     */
    static clockDir(pointList: SPoint[]): number {
        if (pointList.length < 3) {
            return 0;
        }

        // 找出x坐标最大的值
        let index = 0;
        let len = pointList.length;
        for (let i = 1; i < len; i++) {
            if (pointList[i].x > pointList[index].x) {
                index = i;
            }
        }

        let p1 = pointList[index == 0 ? len - 1 : index - 1];
        let p2 = pointList[index];
        let p3 = pointList[(index + 1) % len];
        return (p2.x - p1.x) * (p3.y - p2.y) - (p2.y - p1.y) * (p3.x - p2.x);
    }
}
