/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SPoint } from "@persagy-web/draw";
import { SGraphCommand, SGraphItem, SGraphScene } from "@persagy-web/graph/";

/**
 * 多边形、折线等相关顶点的位置插入命令
 *
 * @author 韩耀龙 <han_yao_long@163.com>
 */
export class SGraphPointListInsert extends SGraphCommand {
    /** 命令名称 */
    readonly command: string;
    /** 指向 item 对象 */
    item: SGraphItem;
    /** 当前位置 */
    pos: SPoint;
    /** 索引 */
    index: number | null;
    /** 顶点数组 */
    pointList: SPoint[];

    /**
     * 构造函数
     *
     * @param scene       item 所在场景
     * @param item        指向 item 对象
     * @param pointList   顶点数组
     * @param pos         当前位置
     * @param index       索引
     */
    constructor(
        scene: SGraphScene,
        item: SGraphItem,
        pointList: SPoint[],
        pos: SPoint,
        index: number | null = null
    ) {
        super(scene);
        this.item = item;
        this.pos = pos;
        this.index = index;
        this.pointList = pointList;
        this.command = "SGraphPointListInsert";
        this.desc = `添加折点=${item.id}`;
    }

    /**
     * 执行重做操作执行
     */
    redo(): void {
        const point = new SPoint(this.pos.x, this.pos.y);
        if (this.index == null) {
            this.pointList.push(point);
        } else {
            this.pointList.splice(this.index, 0, point);
        }

        this.item.update();
    }

    /**
     * 执行取消操作执行
     */
    undo(): void {
        if (this.index == null) {
            this.pointList.pop();
        } else {
            this.pointList.splice(this.index, 1);
        }

        this.item.update();
    }

    /**
     * 命令细节描述
     *
     * @return 字符串类型数据
     */
    toString(): string {
        const pointList = `pointList=${JSON.stringify(this.pointList)}`;
        const pos = `pos=${JSON.stringify(this.pos)}`;
        const index = `index=${this.index}`;
        return `${index};\n${pos};\n${pointList}`;
    }
}
