/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SGraphCommand } from "./SGraphCommand";
import { SGraphItem } from "@persagy-web/graph/";
import { SGraphScene } from "@persagy-web/graph/";

/**
 * 属性修改命令类
 *
 * @author 韩耀龙 <han_yao_long@163.com>
 */
export class SGraphPropertyCommand<T> extends SGraphCommand {
    /** 命令名称 */
    readonly command: string;
    /**	指向 item 对象 */
    item: SGraphItem;
    /** 属性名称 */
    propName: string;
    /** 属性旧值 */
    oldProp: T;
    /** 属性新值 */
    newProp: T;

    /**
     * 构造函数
     *
     * @param scene       命令所属的场景类
     * @param item        命令所属的 item 类
     * @param propName    修改的属性名称
     * @param oldProp     修改前的属性值
     * @param newProp     修改后的属性值
     */
    constructor(
        scene: SGraphScene,
        item: SGraphItem,
        propName: string,
        oldProp: T,
        newProp: T
    ) {
        super(scene);
        this.item = item;
        this.propName = propName;
        this.oldProp = oldProp;
        this.newProp = newProp;
        this.command = "SGraphPropertyCommand";
        this.command = `更新属性=${item.id}`;
    }

    /**
     * redo 操作
     */
    redo(): void {
        // @ts-ignore
        this.item[this.propName] = this.newProp;
        this.item.update();
    }

    /**
     * undo 操作
     */
    undo(): void {
        // @ts-ignore
        this.item[this.propName] = this.oldProp;
        this.item.update();
    }

    /**
     * 命令细节描述
     *
     * @return 字符串型数据
     */
    toString(): string {
        const propName = `propName=${this.propName}`;
        const oldProp = `oldProp=${JSON.stringify(this.oldProp)}`;
        const newProp = `newProp=${JSON.stringify(this.newProp)}`;
        return `${propName};\n${oldProp};\n${newProp}`;
    }
}
