/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SGraphItem } from "../SGraphItem";
import {
    SPainter,
    SPath,
    SPoint,
    SPolygonUtil,
    SRect
} from "@persagy-web/draw/lib";
import { SGraphShape } from "./SGraphShape";
import { AreaGroup } from "../types/AreaGroup";

/**
 * 区域组
 *
 * @author  郝建龙 <haojianlong@sagacloud.cn>
 */
export class SGraphAreaGroupItem extends SGraphShape {
    /** X 坐标最小值 */
    private minX = Number.MAX_SAFE_INTEGER;
    /** X 坐标最大值 */
    private maxX = Number.MIN_SAFE_INTEGER;
    /** Y 坐标最小值 */
    private minY = Number.MAX_SAFE_INTEGER;
    /** Y 坐标最大值 */
    private maxY = Number.MIN_SAFE_INTEGER;
    /** 墙轮廓线坐标 list */
    private readonly pointList: SPoint[][][] = [];
    /** path 对象 */
    private pathList: SPath[] = [];

    /**
     * 构造函数
     *
     * @param parent  父类
     * @param data    轮廓线及风格数据
     */
    constructor(parent: SGraphItem | null, data: AreaGroup) {
        super(parent, data.style);
        if (data.outline.length && data.outline[0] && data.outline[0].length) {
            let tempArr = data.outline;
            this.minX = tempArr[0][0][0].x;
            this.maxX = this.minX;
            this.minY = tempArr[0][0][0].y;
            this.maxY = this.minY;
            // 处理轮廓点数组,同时计算最大最小值
            this.pointList = tempArr.map((t): SPoint[][] => {
                let sPath = new SPath();
                let tempArr = t.map((it): SPoint[] => {
                    let array = it.map(
                        (item): SPoint => {
                            let x = item.x,
                                y = item.y;
                            if (x < this.minX) {
                                this.minX = x;
                            }
                            if (y < this.minY) {
                                this.minY = y;
                            }
                            if (x > this.maxX) {
                                this.maxX = x;
                            }
                            if (y > this.maxY) {
                                this.maxY = y;
                            }
                            return new SPoint(x, y);
                        }
                    );
                    sPath.polygon(array);
                    return array;
                });
                this.pathList.push(sPath);
                return tempArr;
            });
        }
    }

    /**
     * Item 对象边界区域
     *
     * @return 对象边界区域
     */
    boundingRect(): SRect {
        return new SRect(
            this.minX,
            this.minY,
            this.maxX - this.minX,
            this.maxY - this.minY
        );
    }

    /**
     * 判断点是否在区域内
     *
     * @param x 点 x 坐标
     * @param y 点 y 坐标
     * @return 是否在区域内
     */
    contains(x: number, y: number): boolean {
        for (let j = 0; j < this.pointList.length; j++) {
            let arr = this.pointList[j];
            if (arr.length < 1 || !SPolygonUtil.pointIn(x, y, arr[0])) {
                continue;
            }
            if (arr.length == 1) {
                return true;
            }
            let flag = false;
            for (let i = 1; i < arr.length; i++) {
                if (SPolygonUtil.pointIn(x, y, arr[i])) {
                    flag = true;
                    break;
                }
            }
            if (flag) {
                continue;
            }
            return true;
        }
        return false;
    }

    /**
     * Item 绘制操作
     *
     * @param painter    绘制对象
     */
    onDraw(painter: SPainter): void {
        super.onDraw(painter);
        this.pathList.forEach((t): void => {
            painter.drawPath(t);
        });
    }
}
