/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SColor, SPainter, SPen, SRect } from "@persagy-web/draw";
import { SGraphItem } from "../SGraphItem";

/**
 * 线 Item 类
 *
 * @author 庞利祥 <sybotan@126.com>
 */
export class SGraphLineItem extends SGraphItem {
    /** 点 1 的 x 坐标 */
    x1: number;
    /** 点 1 的 y 坐标 */
    y1: number;
    /** 点 2 的 x 坐标 */
    x2: number;
    /** 点 2 的 y 坐标 */
    y2: number;
    /** 线条颜色 */
    color: SColor = SColor.Black;
    /** 线条宽度 */
    width: number = 1;

    /**
     * 构造函数
     *
     * @param x1        线的起始 x 坐标
     * @param y1        线的起始 y 坐标
     * @param x2        线的终止 x 坐标
     * @param y2        线的终止 y 坐标
     * @param width     线的宽度
     * @param color     线的颜色
     * @param parent    是否为虚线
     */
    constructor(
        parent: SGraphItem | null,
        x1: number,
        y1: number,
        x2: number,
        y2: number,
        color: SColor = SColor.Black,
        width: number = 1
    ) {
        super(parent);
        this.x1 = x1;
        this.y1 = y1;
        this.x2 = x2;
        this.y2 = y2;
    }

    /**
     * Item 对象边界区域
     *
     * @return 边界区域
     */
    boundingRect(): SRect {
        let minX = Math.min(this.x1, this.x2);
        let minY = Math.min(this.y1, this.y2);
        let maxX = Math.max(this.x1, this.x2);
        let maxY = Math.max(this.y1, this.y2);
        return new SRect(
            minX - this.width / 2,
            minY - this.width / 2,
            maxX - minX + this.width,
            maxY - minY + this.width
        );
    }

    /**
     * 绘制 Item
     *
     * @param painter 绘制对象
     */
    onDraw(painter: SPainter): void {
        painter.pen = new SPen(this.color, this.width);
        painter.drawLine(this.x1, this.y1, this.x2, this.y2);
    }
}
