/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SGraphShape } from "./SGraphShape";
import { SGraphItem } from "../SGraphItem";
import { SPainter, SRect } from "@persagy-web/draw";
import { SGraphRect } from "..";

/**
 * 矩形(包含圆角矩形) item
 *
 * @author 郝洁 <haojie@persagy.com>
 */
export class SGraphRectItem extends SGraphShape {
    /** item 宽 */
    private _width: number = 0;
    get width(): number {
        return this._width;
    }
    set width(v: number) {
        if (v == this._width) {
            return;
        }
        this._width = v;
        this.update();
    }
    /** item 高 */
    private _height: number = 0;
    get height(): number {
        return this._height;
    }
    set height(v: number) {
        if (v == this._height) {
            return;
        }
        this._height = v;
        this.update();
    }
    /** 绘制矩形的圆角半径 */
    private _radius: number = 0;
    get radius(): number {
        return this._radius;
    }
    set radius(v: number) {
        if (v == this._radius) {
            return;
        }
        this._radius = v;
        this.update();
    }

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      矩形数据
     */
    constructor(parent: SGraphItem | null, data: SGraphRect) {
        super(parent, data.style);
        this.width = data.width;
        this.height = data.height;
        this.moveTo(data.x, data.y);
        if (data.radius && !isNaN(data.radius)) {
            this.radius = Number(data.radius);
        }
    }

    /**
     * Item 对象边界区域
     *
     * @return	边界区域
     */
    boundingRect(): SRect {
        return new SRect(0, 0, this.width, this.height);
    }

    /**
     * 绘制
     *
     * @param painter   绘制对象
     */
    onDraw(painter: SPainter): void {
        super.onDraw(painter);
        if (this.radius != 0) {
            painter.drawRoundRect(0, 0, this.width, this.height, this.radius);
        } else {
            painter.drawRect(0, 0, this.width, this.height);
        }
    }
}
