/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SGraphItem } from "../SGraphItem";
import { SPainter } from "@persagy-web/draw";
import { Style } from "..";
import {
    SColor,
    SGradient,
    SLinearGradient,
    SRadialGradient
} from "@persagy-web/draw/lib";

/**
 * 图形 item 基类
 *
 * @author  郝建龙 <haojianlong@sagacloud.cn>
 */
export class SGraphShape extends SGraphItem {
    /** 当前 item 的风格 */
    style: Style | undefined;

    /**
     * 构造函数
     *
     * @param parent  父类
     * @param style   item 各种状态的样式
     */
    constructor(parent: SGraphItem | null, style?: Style) {
        super(parent);
        this.style = style;
    }

    /**
     * 绘制前设置绘制样式
     *
     * @return 当前 item 状态值
     */
    setStyle(): string {
        let status = "default";
        if (this.style) {
            if (this.enabled) {
                // 是否选中
                if (this.selected) {
                    status = "selected";
                }
            } else {
                status = "disabled";
            }
        }
        return status;
    }

    /**
     * 解析渐变样式
     *
     * @param str   渐变存储的字符串
     * @return 渐变颜色转为的字符串
     */
    parseFill(str: string): SGradient | undefined {
        const index1 = str.indexOf("{");
        const index2 = str.indexOf("}");
        const temp = str.substring(index1 + 1, index2 - 1);
        const dataArr = temp.split(";");
        if (str.indexOf("SLinearGradient") > -1) {
            try {
                // @ts-ignore
                const grad = new SLinearGradient(
                    ...dataArr[0].split(","),
                    // @ts-ignore
                    ...dataArr[1].split(",")
                );
                for (let i = 2; i < dataArr.length; i++) {
                    const cur = dataArr[i].split(",");
                    grad.addColorStop(Number(cur[0]), new SColor(cur[1]));
                }
                return grad;
            } catch (e) {
                console.log("渐变颜色格式有误", e, str);
            }
        } else if (str.indexOf("SRadialGradient") > -1) {
            try {
                // @ts-ignore
                const grad = new SRadialGradient(
                    ...dataArr[0].split(","),
                    // @ts-ignore
                    ...dataArr[1].split(",")
                );
                for (let i = 2; i < dataArr.length; i++) {
                    const cur = dataArr[i].split(",");
                    grad.addColorStop(Number(cur[0]), new SColor(cur[1]));
                }
                return grad;
            } catch (e) {
                console.log("渐变颜色格式有误", e, str);
            }
        }
        return undefined;
    } // parseFill()

    /**
     * 绘制
     *
     * @param painter   绘制对象
     */
    onDraw(painter: SPainter): void {
        const status = this.setStyle();
        if (this.style && this.style[status]) {
            const grad = this.parseFill(this.style[status].fill);
            if (grad) {
                painter.brush.gradient = grad;
            } else {
                painter.brush.color = new SColor(this.style[status].fill);
            }
            painter.pen.color = new SColor(this.style[status].stroke);
            painter.pen.lineWidth = this.style[status].lineWidth;
        } else {
            painter.brush.color = SColor.White;
        }
    }
}
