/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SGraphShape } from "./SGraphShape";
import { SPainter, SRect } from "@persagy-web/draw/lib";
import { SGraphItem } from "../SGraphItem";
import { SGraphSvg } from "../types/SGraphSvg";

/**
 * SVG 图片 item
 *
 * @author  郝建龙 <haojianlong@sagacloud.cn>
 */
export class SGraphSvgItem extends SGraphShape {
    /** 图片 dom */
    img: CanvasImageSource | undefined;
    /** 图片加载是否完成 */
    isLoadOver: boolean = false;
    /** 图片地址 */
    private _url: string = "";
    get url(): string {
        return this._url;
    }
    set url(v: string) {
        if (this._url == v) {
            return;
        }
        this._url = v;
        if (!this.img) {
            this.img = new Image();
        }
        // @ts-ignore
        this.img.src = v;
    }
    /** item 宽 */
    private _width: number = 0;
    get width(): number {
        return this._width;
    }
    set width(v: number) {
        if (v == this._width) {
            return;
        }
        this._width = v;
        this.update();
    }
    /** item 高 */
    private _height: number = 0;
    get height(): number {
        return this._height;
    }
    set height(v: number) {
        if (v == this._height) {
            return;
        }
        this._height = v;
        this.update();
    }

    /** img 宽 */
    private imgWidth: number = 0;
    /** img 高 */
    private imgHeight: number = 0;

    /**
     * 构造函数
     *
     * @param parent  父类
     * @param data    传入数据
     */
    constructor(parent: SGraphItem | null, data: SGraphSvg) {
        super(parent, data.style);
        if (data.url) {
            this.img = new Image();
            this.url = data.url;
            this.img.onload = (e): void => {
                this.isLoadOver = true;
                this.update();
            };
            this.img.onerror = (e): void => {
                this.isLoadOver = false;
                this.update();
                console.log("加载图片错误！", e);
            };
        }
        this.width = data.width;
        this.height = data.height;
        this.moveTo(data.x, data.y);
    }

    /**
     * Item 对象边界区域
     *
     * @return 边界区域
     */
    boundingRect(): SRect {
        return new SRect(0, 0, this.width, this.height);
    }

    /**
     * Item 绘制操作
     *
     * @param painter   绘制对象
     */
    onDraw(painter: SPainter): void {
        if (this.isLoadOver) {
            // @ts-ignore
            painter.drawImage(this.img, 0, 0, this.width, this.height);
        }
    }
}
